@@ -77,7 +77,7 @@ def draw_terrain(s, with_polygons=False):
 	if(with_polygons):
 		#4- terrain polygons
 		for soil_param_set in terrain.soil_params:
-			if not s.terrain._geojson_polygon_offset_applied:
+			if not s.terrain._geojson_polygon_soil_offset_applied:
 				poly=shapely.affinity.translate(soil_param_set["shapely_polygon"],xoff=-raster.xllcorner,yoff=-raster.yllcorner)
 			else:
 				poly=soil_param_set["shapely_polygon"]
@@ -192,10 +192,9 @@ def draw_sample_trajectories(s,nb=50,use_plotly=False):
 		return [ draw_rock_trajectory(s,i,use_plotly=True) for i in rocks_to_plot ]
 
 def get_plotly_raw_html(s, sample_trajectories=False):
-	#TODO : trees
-	
 	#0- Initialize the lists
 	data = []
+	shapes = []
 	updatemenus=[]
 	
 	#1- plot the terrain:
@@ -224,7 +223,7 @@ def get_plotly_raw_html(s, sample_trajectories=False):
 			polygons=[polygons]
 			multiPoly=True
 		for polygon_number,p in enumerate(polygons):
-			if not s.terrain._geojson_polygon_offset_applied:
+			if not s.terrain._geojson_polygon_soil_offset_applied:
 				p=shapely.affinity.translate(p,xoff=-s.terrain.Z_raster.xllcorner,yoff=-s.terrain.Z_raster.yllcorner)
 			data.append(
 				plygo.Scatter(
@@ -243,6 +242,32 @@ def get_plotly_raw_html(s, sample_trajectories=False):
 					showlegend=True if polygon_number==0 else False
 				)
 			)
+		# forest params
+	for param_set_number,forest_param_set in enumerate(s.terrain.forest_params):
+		polygons=forest_param_set["shapely_polygon"]
+		multiPoly=True
+		if isinstance(polygons,shapely.geometry.Polygon): #to handle polygons and multipolygons
+			polygons=[polygons]
+			multiPoly=True
+		for polygon_number,p in enumerate(polygons):
+			if not s.terrain._forest_offset_applied:
+				p=shapely.affinity.translate(p,xoff=-s.terrain.Z_raster.xllcorner,yoff=-s.terrain.Z_raster.yllcorner)
+			data.append(
+				plygo.Scatter(
+					name="Forest zone "+str(param_set_number),
+					x=list(p.exterior.xy[0]),
+					y=list(p.exterior.xy[1]),
+					mode='lines',
+					text=forest_param_set["params"].__repr__().replace("'","").replace("{","").replace("}","").replace(", ","<br>"),
+					line=dict(
+						color=cp.color_to_html(forest_param_set["color"]),
+						width=2,
+						dash='dot',
+					),
+					legendgroup="Forest zone "+str(param_set_number),
+					showlegend=True if polygon_number==0 else False
+				)
+			)
 	
 		# rocks start params:
 	for param_set_number,start_param_set in enumerate(s.rocks_start_params):
@@ -252,7 +277,7 @@ def get_plotly_raw_html(s, sample_trajectories=False):
 			polygons=[polygons]
 			multiPoly=True
 		for polygon_number,p in enumerate(polygons):
-			if not s.terrain._geojson_polygon_offset_applied:
+			if not s.terrain._geojson_polygon_soil_offset_applied:
 				p=shapely.affinity.translate(p,xoff=-s.terrain.Z_raster.xllcorner,yoff=-s.terrain.Z_raster.yllcorner)
 			data.append(
 				plygo.Scatter(
@@ -336,7 +361,12 @@ def get_plotly_raw_html(s, sample_trajectories=False):
 				)
 			)
 	
-	#5- optionnaly show raster fields results:
+	#5- optionnaly show trees:
+	if len(s.terrain.trees):
+		kwargs = {'type': 'circle', 'xref': 'x', 'yref': 'y', 'fillcolor': 'green', "line_width":0}
+		shapes += [plygo.layout.Shape(x0=t[0]-t[2]/2/100., y0=t[1]-t[2]/2/100., x1=t[0]+t[2]/2/100., y1=t[1]+t[2]/2/100., **kwargs) for t in s.terrain.trees_as_array]
+		
+	#6- optionnaly show raster fields results:
 	if s.pp and s.pp.has_run:
 		fields=s.pp.raster.get_scalar_fields()
 		if "Z" in fields: fields.remove("Z")
@@ -409,6 +439,7 @@ def get_plotly_raw_html(s, sample_trajectories=False):
 		),
 		#showlegend= False,
 		hovermode = 'closest',
+		shapes=shapes,
 		#annotations=annotations,
 	)
 		
