@@ -277,7 +277,7 @@ class Parallelepiped(Rock):
 
 from . import ThreeDEnginesToolbox
 class Terrain(Triangulated_Object):
-	valid_input_geojson_attrs=np.array([
+	valid_input_soil_geojson_attrs=np.array([
 		["R_n",					"R<sub>n</sub>",		float,	0,	0.95	],
 		["R_t",					"R<sub>t</sub>",		float,	0,	0.95	],
 		["roughness",			"Roughness",			float,	0,	1.		],
@@ -288,7 +288,14 @@ class Terrain(Triangulated_Object):
 		["mu",					"μ",					float,	0,	0.95	],
 		["mu_r",				"μ<sub>r</sub>",		float,	0,	100		],
 	])
-	def __init__(self,DEM_asc=None,params_geojson=None,default_faces_params={"R_n":0.6, "R_t":0.6, "roughness":0.1,"bounce_model_number":0,"phi":30,"v_half":2.5,"e":0.1,"mu":0.8,"mu_r":0.5}):
+	
+	valid_input_forest_geojson_attrs=np.array([
+		["density",				"Density (trees/ha)",	float,	0,	10000	],
+		["dhp_mean",			"Mean diameter (cm)",	float,	1,	200		],
+		["dhp_std",				"Diameter σ (cm)",		float,	0,	200		],
+	])
+	#NOTE: params_geojson below is deprecated
+	def __init__(self,DEM_asc=None,params_geojson=None,soil_params_geojson=None,forest_params_input_file=None,default_faces_params={"R_n":0.6, "R_t":0.6, "roughness":0.1,"bounce_model_number":0,"phi":30,"v_half":2.5,"e":0.1,"mu":0.8,"mu_r":0.5}):
 		Triangulated_Object.__init__(self)
 		self.enable_points_view=False
 		self.color=[0.68235294, 0.5372549 , 0.39215686]
@@ -299,15 +306,28 @@ class Terrain(Triangulated_Object):
 		self.opacity=1
 		self.faces_xyz_bb=None
 		self.soil_params=[]
+		self.forest_params=[]
 		self.default_faces_params=default_faces_params
-		self._geojson_polygon_offset_applied=False
+		self.automatic_generate_forest={"enable":False, "density":0, "dhp_mean":0, "dhp_std":0, "mode":"terrain"}
+		self._geojson_polygon_soil_offset_applied=False
+		self._forest_offset_applied=False
 		if(DEM_asc is not None):
 			if(platrock.web_ui):	#don't populate/precompute here, do it in s.before_run_tasks()
 				self.set_Z_raster_from_DEM_file(DEM_asc)
-			else:
+			else:	#populate/precompute now 
 				self.from_DEM_file(DEM_asc)
-		if(params_geojson is not None): #NOTE: the function below needs the Z_raster to be set, as it needs the xllcorner/yllcorner data
-			self.set_soil_params_from_geojson(params_geojson)
+		if params_geojson is not None: #retro-compatibility
+			soil_params_geojson=params_geojson
+		if(soil_params_geojson is not None):
+			self.set_soil_params_from_geojson(soil_params_geojson)
+		if(forest_params_input_file is not None):
+			self.automatic_generate_forest["enable"]=True
+			if forest_params_input_file[-8:]==".geojson":
+				self.automatic_generate_forest["mode"]="zones"
+				self.set_forest_params_from_geojson(forest_params_input_file)
+			elif forest_params_input_file[-4:]==".xyd":
+				self.automatic_generate_forest["mode"]="xyd"
+				self.set_forest_params_from_xyd(forest_params_input_file)
 
 	def from_DEM_file(self,filename):
 		self.set_Z_raster_from_DEM_file(filename)
@@ -331,7 +351,7 @@ class Terrain(Triangulated_Object):
 	
 	def set_soil_params_from_geojson(self,params_geojson):
 		try:
-			self._geojson_polygon_offset_applied=False
+			self._geojson_polygon_soil_offset_applied=False
 			shp_file=gdal.OpenEx(params_geojson)
 			self.soil_params=[]
 			for feat in shp_file.GetLayer(0):
@@ -346,7 +366,7 @@ class Terrain(Triangulated_Object):
 					soil_params["bounce_model_number"] = soil_params.pop("bounce_mod")
 				#validate the parameters from the input geojson
 				for k in list(soil_params.keys()):
-					if(k not in Terrain.valid_input_geojson_attrs[:,0]):
+					if(k not in Terrain.valid_input_soil_geojson_attrs[:,0]):
 						soil_params.pop(k)
 				self.soil_params[-1]["params"]=soil_params
 			return 0
@@ -354,15 +374,53 @@ class Terrain(Triangulated_Object):
 			message="Failed to import the geojson:"+str(e)
 			Debug.error(message)
 			return message
+	
+	def set_forest_params_from_geojson(self,params_geojson):
+		try:
+			self._forest_offset_applied=False
+			shp_file=gdal.OpenEx(params_geojson)
+			self.forest_params=[]
+			self.automatic_generate_forest.pop("trees_array",None) #just to avoid input file collide with webui
+			for feat in shp_file.GetLayer(0):
+				feat_dict=feat.ExportToJson(as_object=True)
+				self.forest_params.append({})
+				shapely_poly = shapely.geometry.shape(feat_dict['geometry'])
+				self.forest_params[-1]["shapely_polygon"]=shapely_poly
+				self.forest_params[-1]["color"]=cp.qualitative10[len(self.forest_params)-1]
+				forest_params = feat_dict['properties']
+				#validate the parameters from the input geojson
+				for k in list(forest_params.keys()):
+					if(k not in Terrain.valid_input_forest_geojson_attrs[:,0]):
+						forest_params.pop(k)
+				self.forest_params[-1]["params"]=forest_params
+			return 0
+		except Exception as e:
+			message="Failed to import the geojson:"+str(e)
+			Debug.error(message)
+			return message
+	
+	def set_forest_params_from_xyd(self,xyd_file):
+		try:
+			self.forest_params=[] #just to avoid input file collide with webui
+			self._forest_offset_applied=False
+			with open(xyd_file,'r') as f:
+				trees=np.genfromtxt(f,dtype=float,delimiter=",")
+			self.automatic_generate_forest["trees_array"]=trees
+			self.automatic_generate_forest["mode"]="xyd" #this will disable all automatic generation
+			return 0
+		except Exception as e:
+			message="Failed to import the xyd:"+str(e)
+			Debug.error(message)
+			return message
 		
 	def set_faces_params_from_geojson(self):
 		#loop on faces to set attrbutes
 		for face in self.faces:	#FIXME: this loop can be slow, optimize by replacing shapely "contains" by a cython or numpy array algo ?
 			shapely_point=shapely.geometry.Point(face.cog[0:2])
 			for soil_param in self.soil_params:
-					if(soil_param["shapely_polygon"].contains(shapely_point)):
-						face.__dict__.update(soil_param["params"])	#update will overwrite params
-						break
+				if(soil_param["shapely_polygon"].contains(shapely_point)):
+					face.__dict__.update(soil_param["params"])	#update will overwrite params
+					break
 	
 	def populate_from_Z_raster(self):
 			points_ids=np.ones([self.Z_raster.nx,self.Z_raster.ny])*-1 #this is a map of points ids, useful to link [x,y] cells to corresponding point ids.
@@ -404,11 +462,6 @@ class Terrain(Triangulated_Object):
 	
 	def set_faces_normals_and_cogs(self):
 		ThreeDEnginesToolbox.set_faces_normals_and_cogs(self.faces)
-		#for f in self.faces:
-			#normal=((f.points[0].pos-f.points[1].pos).cross(f.points[2].pos-f.points[1].pos)).normalized()
-			#if(normal[2]<0.):normal*=-1.
-			#f.normal=normal
-			#f.cog=(f.points[0].pos + f.points[1].pos + f.points[2].pos)/3
 	
 	def set_faces_xyz_bbs(self):
 		self.faces_xyz_bb=np.asarray([f.xyz_bb for f in self.faces])
@@ -418,8 +471,8 @@ class Terrain(Triangulated_Object):
 		self.vertical_surface=0
 		for f in self.faces:
 			self.vertical_surface += 0.5*np.abs(Math.cross2(f.points[1].pos[0:2]-f.points[0].pos[0:2],f.points[2].pos[0:2]-f.points[0].pos[0:2]))
-			
-	def generate_random_forest(self,density,dhp_mean, dhp_std):
+	
+	def generate_random_forest(self,density,dhp_mean,dhp_std):
 		self.set_vertical_surface()
 		nb_trees=int(density*self.vertical_surface/10000) #NOTE: density is in trees/ha, vertical_surface is in m^2
 		nb_faces=len(self.faces)
@@ -434,6 +487,28 @@ class Terrain(Triangulated_Object):
 			dhp = Math.get_random_value_from_gamma_distribution(dhp_mean,dhp_std)
 			self.trees.append(Tree(pos=rand_point,dhp=dhp))
 	
+	def generate_forest(self):
+		mode=self.automatic_generate_forest["mode"]
+		if mode=="terrain": #forest everywhere
+			self.generate_random_forest(self.automatic_generate_forest["density"],self.automatic_generate_forest["dhp_mean"],self.automatic_generate_forest["dhp_std"])
+		elif mode=="zones":#forest in geojson polygons
+			for forest_zone in self.forest_params: #loop on polygons of parameters
+				poly=forest_zone["shapely_polygon"]
+				minX, minY, maxX, maxY = poly.bounds
+				nb_trees=int(forest_zone["params"]["density"]*poly.area/10000) #NOTE: density is in trees/ha, vertical_surface is in m^2
+				#basic rejection algorithm:
+				t=0
+				while t<nb_trees:
+					tryX=np.random.random()*(maxX-minX)+minX
+					tryY=np.random.random()*(maxY-minY)+minY
+					if poly.contains(shapely.geometry.Point(tryX,tryY)):
+						dhp = Math.get_random_value_from_gamma_distribution(forest_zone["params"]["dhp_mean"],forest_zone["params"]["dhp_std"])
+						self.trees.append(Tree(pos=[tryX,tryY],dhp=dhp))
+						t+=1
+		elif mode=="xyd":#forest in xyd file
+			for t in self.automatic_generate_forest["trees_array"]:
+				self.trees.append(Tree(pos=[t[0],t[1]],dhp=t[2]))
+	
 	def get_vtk_string(self): #NOTE: deprecated
 		import vtk
 		points = vtk.vtkPoints()
