@@ -76,7 +76,7 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		enable_GUI (bool): enables the 3D view (experimental)
 	"""
 	webui_typename="PlatRock 3D"
-	valid_input_geojson_attrs=np.array([
+	valid_input_rocks_geojson_attrs=np.array([
 		["number",			"Rocks count",							int,	1,		1000	],
 		["z",				"Drop height (m)",						float,	2,		50		],
 		["rocks_min_vol",	"Min rocks volume (m<sup>3</sup>)",		float,	0.1,	10		],
@@ -106,7 +106,6 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		if(checkpoints_geojson is not None):
 			self.set_checkpoints_from_geojson(checkpoints_geojson)
 		self.automatic_engines_fill=None
-		self.automatic_generate_forest={"enable":False, "density":0, "dhp_mean":30, "dhp_std":10}
 		self.limit_fps=30 #used only with 3D view
 		self._geojson_polygon_offset_applied=False
 		self.pp=None #this will be set to Postprocessings.Postprocessing(self) later
@@ -134,7 +133,7 @@ class Simulation(platrock.Common.Simulations.Simulation):
 				self.rocks_start_params[-1]["color"]=cp.dark2[len(self.rocks_start_params)-1]
 				rocks_properties = feat_dict['properties']
 				for k in list(rocks_properties.keys()):
-					if(k not in Simulation.valid_input_geojson_attrs[:,0]):
+					if(k not in Simulation.valid_input_rocks_geojson_attrs[:,0]):
 						rocks_properties.pop(k)
 				self.rocks_start_params[-1]["params"]=rocks_properties
 				#set the simulation total number of rocks
@@ -321,10 +320,19 @@ class Simulation(platrock.Common.Simulations.Simulation):
 				rsp["shapely_polygon"] = shapely.affinity.translate(rsp["shapely_polygon"],xoff=-self.terrain.Z_raster.xllcorner,yoff=-self.terrain.Z_raster.yllcorner)
 			self._geojson_polygon_offset_applied=True
 		#Offset the soil params polygons:
-		if not self.terrain._geojson_polygon_offset_applied:
+		if not self.terrain._geojson_polygon_soil_offset_applied:
 			for sp in self.terrain.soil_params:
 				sp["shapely_polygon"] = shapely.affinity.translate(sp["shapely_polygon"],xoff=-self.terrain.Z_raster.xllcorner,yoff=-self.terrain.Z_raster.yllcorner)
-			self.terrain._geojson_polygon_offset_applied=True
+			self.terrain._geojson_polygon_soil_offset_applied=True
+		#Offset the forest params polygons:
+		if not self.terrain._forest_offset_applied:
+			for sp in self.terrain.forest_params:
+				sp["shapely_polygon"] = shapely.affinity.translate(sp["shapely_polygon"],xoff=-self.terrain.Z_raster.xllcorner,yoff=-self.terrain.Z_raster.yllcorner)
+			if "trees_array" in self.terrain.automatic_generate_forest.keys(): #case of xyd input
+				trees_array=self.terrain.automatic_generate_forest["trees_array"]
+				trees_array[:,0]-=self.terrain.Z_raster.xllcorner
+				trees_array[:,1]-=self.terrain.Z_raster.yllcorner
+			self.terrain._forest_offset_applied=True
 		#call the parent function, which needs nb_rocks to be initialized:
 		super(Simulation, self).before_run_tasks()
 		
@@ -334,7 +342,7 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		if(platrock.web_ui):
 			self.terrain.populate_from_Z_raster()
 			self.terrain.precompute_datas()
-			self.automatic_generate_forest["enable"]=True
+			self.terrain.automatic_generate_forest["enable"]=True
 			self.__init_arrays__()
 		self.terrain.set_faces_params_from_geojson()
 		if(self.automatic_engines_fill=="builtin"):
@@ -346,8 +354,8 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		elif(self.automatic_engines_fill=="siconos"):
 			self.engines=[	Engines.Siconos(terrain=self.terrain, dt=0.001)]
 		
-		if(self.automatic_generate_forest["enable"]):
-			self.terrain.generate_random_forest(self.automatic_generate_forest["density"],self.automatic_generate_forest["dhp_mean"],self.automatic_generate_forest["dhp_std"])
+		if(self.terrain.automatic_generate_forest["enable"]):
+			self.terrain.generate_forest()
 			self.enable_forest=len(self.terrain.trees)>0
 		if self.enable_forest:
 			self.terrain.trees_as_array=np.asarray([np.append(t.pos,t.dhp) for t in self.terrain.trees])
