@@ -77,14 +77,18 @@ class Simulation(platrock.Common.Simulations.Simulation):
 	webui_typename="PlatRock 2D"
 	valid_rocks_start_params=np.array([
 		["nb_rocks",		"Rocks count",						int,		1,		10000	],
-		["rocks_volume",	"Rocks volume (m<sup>3</sup>)",		float,		0.1,	100.	],
+		["rocks_min_vol",	"Min rocks volume (m<sup>3</sup>)",	float,		0.1,	100.	],
+		["rocks_max_vol",	"Max rocks volume (m<sup>3</sup>)",	float,		0.1,	100.	],
 		["rocks_vx",		"Horizontal velocity (m/s)",		float,		-10,	10,		],
 		["rocks_vz",		"Vertical velocity (m/s)",			float,		-10,	10,		],
 		["rocks_angVel",	"Angular velocity (rad/s)",			float,		-6.3,	6.3,	],
-		["rocks_x",			"Horizontal position (m)",			float,		0 ,		np.inf,	],
-		["rocks_z",			"Falling height (m)",				float,		0.1,	50,		],
+		["rocks_min_x",		"Min horizontal position (m)",		float,		0 ,		np.inf,	],
+		["rocks_max_x",		"Max horizontal position (m)",		float,		0 ,		np.inf,	],
+		["rocks_min_z",		"Min falling height (m)",			float,		0.1,	50,		],
+		["rocks_max_z",		"Max falling height (m)",			float,		0.1,	50,		],
 		["rocks_density",	"Density (kg/m<sup>3</sup>)",		float,		500,	5000,	]
 	])
+	retro_compatibility_params=["rocks_volume","rocks_x","rocks_z"]
 	
 	def __init__(self, checkpoints_x=[], rocks_start_params={}, **kwargs):
 		super(Simulation, self).__init__(**kwargs)
@@ -99,19 +103,39 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		
 		# SET THE ROCKS DEFAULT PARAMS :
 		self.nb_rocks=100
-		self.rocks_volume=2.
 		self.rocks_density=2650.
-		self.rocks_x=1.
-		self.rocks_z=5.
+		self.rocks_min_x=1.
+		self.rocks_max_x=2.
+		self.rocks_min_z=5.
+		self.rocks_max_z=8.
 		self.rocks_vx=1.
 		self.rocks_vz=0.
 		self.rocks_angVel=0.
+		self.rocks_min_vol=1.
+		self.rocks_max_vol=2.
 		#self.rocks_start_params=rocks_start_params
 		#OVERRIDE THE DEFAULT PARAMS, IF AVAILABLE IN rocks_start_params dict.
 		for k in rocks_start_params.keys():
 			if(k in self.valid_rocks_start_params[:,0]):
 				self.__dict__[k]=rocks_start_params[k]
 		self.override_mu_r=30
+		self.apply_retro_compatibility(rocks_start_params)
+		self.__init_arrays__()
+	
+	def __init_arrays__(self):
+		#Handle generated arrays based on params. This method is run at simulation instanciation and during before_run_tasks in case of web_ui usage: this allows script users to manually change it.
+		if hasattr(self,"rocks_volume"):
+			self.rocks_volumes=[self.rocks_volume]*self.nb_rocks
+		else:
+			self.rocks_volumes=self.random_generator.rand(self.nb_rocks)*(self.rocks_max_vol - self.rocks_min_vol)+self.rocks_min_vol
+		if hasattr(self,"rocks_x"):
+			self.rocks_start_x=[self.rocks_x]*self.nb_rocks
+		else:
+			self.rocks_start_x=self.random_generator.rand(self.nb_rocks)*(self.rocks_max_x - self.rocks_min_x)+self.rocks_min_x
+		if hasattr(self,"rocks_z"):
+			self.rocks_start_z=[self.rocks_z]*self.nb_rocks
+		else:
+			self.rocks_start_z=self.random_generator.rand(self.nb_rocks)*(self.rocks_max_z - self.rocks_min_z)+self.rocks_min_z
 		
 	def save_to_file(self):
 		"""
@@ -239,18 +263,20 @@ class Simulation(platrock.Common.Simulations.Simulation):
 			self.enable_forest=False
 		if(self.enable_forest and self.forest_impact_model==None):
 			self.forest_impact_model=BounceModels.Toe_Tree()
+		if platrock.web_ui:
+			self.__init_arrays__()
 	
 	def before_rock_launch_tasks(self):
 		super(Simulation, self).before_rock_launch_tasks()
-		self.current_rock=Objects.Rock(x=self.rocks_x, height=self.rocks_z, vel=Math.Vector2([self.rocks_vx,self.rocks_vz]), angVel=self.rocks_angVel, volume=self.rocks_volume, density=self.rocks_density)
+		self.current_rock=Objects.Rock(x=self.rocks_start_x[self.current_rock_number], height=self.rocks_start_z[self.current_rock_number], vel=Math.Vector2([self.rocks_vx,self.rocks_vz]), angVel=self.rocks_angVel, volume=self.rocks_volumes[self.current_rock_number], density=self.rocks_density)
 		self.current_rock.update_current_segment(self.terrain)
 		self.current_rock.pos[1]+=self.current_rock.current_segment.get_y(self.current_rock.pos[0])
 		self.output.add_rock(self.current_rock)
 		self.output.add_contact(self.current_rock,Math.Vector2([0.,1.]),Out.START) #store the rock initial position
 		Debug.Print("New rock:",self.current_rock.pos,self.current_rock.vel)
 	
-	def after_rock_propagation_tasks(self):
-		type(self).__bases__[0].after_rock_propagation_tasks(self)
+	def after_rock_propagation_tasks(self,*args,**kwargs):
+		type(self).__bases__[0].after_rock_propagation_tasks(self,*args,**kwargs)
 		if(self.current_rock.out_of_bounds):
 			if (not platrock.SICONOS_FOUND) or (type(self)!=platrock.TwoDShape.Simulations.Simulation):
 				#Fly into the void...
@@ -266,8 +292,9 @@ class Simulation(platrock.Common.Simulations.Simulation):
 			#Mark the stop position
 			self.output.add_contact(self.current_rock,Math.Vector2([0.,0.]),Out.STOP)
 	
-	def after_successful_run_tasks(self):
-		type(self).__bases__[0].after_successful_run_tasks(self)
+	def after_successful_run_tasks(self,*args,**kwargs):
+		# The following trick is used to trigger the parent method with the "friend class" mecanism (as this class is "imported" by other 2D simulation types).
+		type(self).__bases__[0].after_successful_run_tasks(self,*args,**kwargs)
 		self.update_checkpoints()
 		if platrock.web_ui:
 			import platrock.GUI.Plot2D as Plot2D
